package org.openjdk.sound;

import java.util.ArrayList;
import java.util.List;

import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Control;
import javax.sound.sampled.Line;
import javax.sound.sampled.LineListener;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.Control.Type;

public class PulseMixer implements javax.sound.sampled.Mixer {
	// singleton
	
	private static PulseMixer _instance = null;
	
	private boolean isOpen = false;
	
	private List<PulseSourceDataLine> _sourceLines = null;
	private List<PulseTargetDataLine> _targetLines = null;
	
	private Line.Info _sourceDataLineInfo = new Line.Info(PulseSourceDataLine.class);
	private Line.Info _targetDataLineInfo = new Line.Info(PulseTargetDataLine.class);
	
	private PulseMixer(){
		
	}
	
	synchronized public static PulseMixer getInstance() {
		if ( _instance == null) {
			_instance = new PulseMixer();
		}
		return _instance;
	}
	
	@Override
	public Line getLine(javax.sound.sampled.Line.Info info)
			throws LineUnavailableException {
		
		if (!isOpen) {
			throw new LineUnavailableException();
		}
		
		if ( info.matches(_sourceDataLineInfo)) {
			PulseSourceDataLine sourceLine = new PulseSourceDataLine();
			_sourceLines.add(sourceLine);
			return sourceLine;
		}
		
		if (info.matches(_targetDataLineInfo)) {
			PulseTargetDataLine targetLine = new PulseTargetDataLine();
			_targetLines.add(targetLine);
			return targetLine;
		}
		
		throw new IllegalArgumentException();
	}

	@Override
	public int getMaxLines(javax.sound.sampled.Line.Info info) {
		return AudioSystem.NOT_SPECIFIED;
	}

	@Override
	public Info getMixerInfo() {
		return PulseMixerInfo.getInfo();
	}

	@Override
	public javax.sound.sampled.Line.Info[] getSourceLineInfo() {
		Line.Info[] info = { new Line.Info(PulseSourceDataLine.class), };
		return info;
	}

	@Override
	public javax.sound.sampled.Line.Info[] getSourceLineInfo(
			javax.sound.sampled.Line.Info info) {
		Line.Info sourceInfo = new Line.Info(PulseSourceDataLine.class);
		if (info.matches(sourceInfo)){
			Line.Info[] sourceInfos = { sourceInfo, };
			return sourceInfos;
		} else {
			Line.Info[] sourceInfos = { };
			return sourceInfos;
			
		}
	}
	@Override
	public Line[] getSourceLines() {
		return (Line[]) _sourceLines.toArray();
		
	}

	@Override
	public javax.sound.sampled.Line.Info[] getTargetLineInfo() {
		Line.Info[] info = { new Line.Info(PulseTargetDataLine.class), };
		return info;
	}

	@Override
	public javax.sound.sampled.Line.Info[] getTargetLineInfo(
			javax.sound.sampled.Line.Info info) {
		Line.Info sourceInfo = new Line.Info(PulseTargetDataLine.class);
		if (info.matches(sourceInfo)){
			Line.Info[] sourceInfos = { sourceInfo, };
			return sourceInfos;
		} else {
			Line.Info[] sourceInfos = { };
			return sourceInfos;	
		}
	}

	@Override
	public Line[] getTargetLines() {
		return (Line[]) _targetLines.toArray();
	}

	@Override
	public boolean isLineSupported(javax.sound.sampled.Line.Info info) {
		if ( _sourceDataLineInfo.matches(info)) {
			return true;
		}
		return false;
	}

	@Override
	public boolean isSynchronizationSupported(Line[] lines, boolean maintainSync) {
		// FIXME
		return false;
	}

	@Override
	public void synchronize(Line[] lines, boolean maintainSync) {
		//FIXME pulse audio supports this
		throw new IllegalArgumentException();
	}

	@Override
	public void unsynchronize(Line[] lines) {
		// FIXME should be able to implement this
		throw new IllegalArgumentException();
		
	}

	@Override
	public void addLineListener(LineListener listener) {
		System.out.println("UNIMPLEMENTED METHOD: PulseMixer.addListener()");
		throw new UnsupportedOperationException("Unimplemented method");
	}

	@Override
	public void close() {
		isOpen = false;
	}

	@Override
	public Control getControl(Type control) {
		return null;
	}

	@Override
	public Control[] getControls() {
		return null;
	}

	@Override
	public javax.sound.sampled.Line.Info getLineInfo() {
		System.out.println("DEBUG: PulseMixer.getLineInfo() called");
		return null;
	}

	@Override
	public boolean isControlSupported(Type control) {
		// FIXME
		return false;
	}

	@Override
	public boolean isOpen() {
		return isOpen;
	}

	@Override 
	public void open() throws LineUnavailableException {
		openLocal();
	}
	
	
	
	public void openLocal() {
		
		_sourceLines = new ArrayList<PulseSourceDataLine>();
		_targetLines = new ArrayList<PulseTargetDataLine>();
		
		
		isOpen = true;
	}
	
	public void openRemote(String url) {
		isOpen = true;
	}

	@Override
	public void removeLineListener(LineListener listener) {
		System.out.println("UNIMPLEMENTED");
		throw new UnsupportedOperationException("Unimplemented method");
	}

}
